Agent skill

Optimize Triton Ascend Grouped Attention Access

by Krusty84 in Krusty84/triton-ascend-agent-dev-kit

Optimize irregular KV-cache access in Triton-Ascend grouped decode attention by vector-loading contiguous inner dimensions, assembling discrete outer rows with CANN slice extensions, and transposing…

Apache-2.0Auto-check passedAI & LLM Engineering

Install Optimize Triton Ascend Grouped Attention Access

skills CLI
$ npx skills add Krusty84/triton-ascend-agent-dev-kit --skill optimize-triton-ascend-grouped-attention-access -a claude-code

Project install by default; add -g for ~/.claude/skills/.

GitHub CLI
$ gh skill install Krusty84/triton-ascend-agent-dev-kit optimize-triton-ascend-grouped-attention-access --agent claude-code

Project scope by default; add --scope user for a personal install. Needs GitHub CLI 2.90.0 or later (public preview).

Manual copy
$ git clone --depth 1 https://github.com/Krusty84/triton-ascend-agent-dev-kit.git skills-src && mkdir -p .claude/skills && cp -r skills-src/skills/optimize-triton-ascend-grouped-attention-access .claude/skills/optimize-triton-ascend-grouped-attention-access && rm -rf skills-src

Use ~/.claude/skills/ instead of .claude/skills for a personal install. The folder must contain SKILL.md.

Claude Code skills documentation · loads skills from .claude/skills/

Facts

Skill name
optimize-triton-ascend-grouped-attention-access
GitHub stars
106
Token cost
~658 tokens
SKILL.md length
207 words
Files
1
Skills in repo
23
Repo updated
First seen
Licence
Apache-2.0

At a glance

Optimize irregular KV-cache access in Triton-Ascend grouped decode attention by vector-loading contiguous inner dimensions, assembling discrete outer rows with CANN slice extensions, and transposing…

  • Works in 6 steps: Classify each target tile by which axis… → Load a discrete outer row and its… → Assemble BLOCK_N rows into an on-chip… → …
  • An agent ports grouped-query decode attention whose K
  • SKILL.md covers Goal, Workflow, Implementation Pattern and Ascend Guardrails, plus 1 more section
  • Instructions only: no scripts, shell commands, URLs or credentials in SKILL.md

What it does

Optimize Triton Ascend Grouped Attention Access is an agent skill from Krusty84/triton-ascend-agent-dev-kit. Optimize irregular KV-cache access in Triton-Ascend grouped decode attention by vector-loading contiguous inner dimensions, assembling discrete outer rows with CANN slice extensions, and transposing UB tiles for tl.dot. Use when an agent ports grouped-query decode attention whose K or V cache addresses are indirect and naive low-axis-discrete loads degrade to scalar memory access.

Its SKILL.md is about 660 tokens, which your agent loads only when the skill is triggered. It is a single SKILL.md file with no bundled scripts.

It sits in AI & LLM Engineering. The repository describes itself as: A toolkit for AI agents used for development on Triton-Ascend for Ascend NPU. The licence is Apache-2.0.

When your agent uses it

  • An agent ports grouped-query decode attention whose K
  • V cache addresses are indirect and naive low-axis-discrete loads degrade to scalar memory access

Example prompts

  • “/optimize-triton-ascend-grouped-attention-access”

Requirements

  • Python 3

Workflow steps

6 steps, taken from the first numbered list in SKILL.md.

  1. Classify each target tile by which axis is discrete and which axis is contiguous.
  2. Load a discrete outer row and its contiguous head dimension as one vector.
  3. Assemble BLOCK_N rows into an on-chip tensor with extension.insert_slice.
  4. Transpose the assembled K tile when tl.dot requires HEAD_DIM by BLOCK_N.
  5. Load V directly as BLOCK_N by VALUE_DIM when its inner value dimension is contiguous.
  6. Feed the tiles into numerically stable online softmax across KV splits.

What it can do on your machine

Read from SKILL.md and the folder at commit 4ab5ee7. It shows what the files ask for, not the result of running them.

  • Tool permissions

    Pre-approves nothing: there is no allowed-tools line, so your agent's usual permission prompts apply.

    From allowed-tools in the SKILL.md frontmatter.

  • Runs code

    No scripts in the folder and no shell commands in SKILL.md (its code samples are python).

    From the folder's file list and the shell code blocks in SKILL.md.

  • Network

    No URLs in SKILL.md.

    From URLs in SKILL.md, links to its own repository left out.

  • Credentials

    Names no API keys, tokens, secrets or passwords.

    From names ending in _API_KEY, _TOKEN, _SECRET, _KEY or _PASSWORD in SKILL.md.

Context cost

Optimize Triton Ascend Grouped Attention Access loads about 658 tokens when it runs. Until then it costs about 108 tokens; SKILL.md has 207 words of instructions outside code blocks.

Always · name and description, kept in context so the agent knows when to use it
~108
When it runs · the whole SKILL.md, loaded when a task matches
~658

Estimates: characters ÷ 4, the usual rule of thumb; real counts depend on the model's tokenizer. Scripts and assets cost tokens only if the agent reads them.

Safety

Auto-check passed

The automated check found no risky patterns in SKILL.md.

Automated static check — not a guarantee. Review scripts before installing. It scans the text of SKILL.md for risky patterns (piping downloads into a shell, reading credential files, hidden Unicode, destructive commands); files beside SKILL.md are not scanned.

SKILL.md

The full file from Krusty84/triton-ascend-agent-dev-kit at commit 4ab5ee7, republished under its Apache-2.0 licence (© Krusty84). 207 words, ~658 tokens.

Download SKILL.mdSave it as .claude/skills/optimize-triton-ascend-grouped-attention-access/SKILL.md (or your agent's skills folder).
name
optimize-triton-ascend-grouped-attention-access
description
Optimize irregular KV-cache access in Triton-Ascend grouped decode attention by vector-loading contiguous inner dimensions, assembling discrete outer rows with CANN slice extensions, and transposing UB tiles for tl.dot. Use when an agent ports grouped-query decode attention whose K or V cache addresses are indirect and naive low-axis-discrete loads degrade to scalar memory access.

Optimize Triton-Ascend Grouped Attention Access

Goal

Preserve vectorized contiguous loads while gathering KV-cache rows selected by an indirect token-location vector.

Workflow

  1. Classify each target tile by which axis is discrete and which axis is contiguous.
  2. Load a discrete outer row and its contiguous head dimension as one vector.
  3. Assemble BLOCK_N rows into an on-chip tensor with extension.insert_slice.
  4. Transpose the assembled K tile when tl.dot requires HEAD_DIM by BLOCK_N.
  5. Load V directly as BLOCK_N by VALUE_DIM when its inner value dimension is contiguous.
  6. Feed the tiles into numerically stable online softmax across KV splits.

Implementation Pattern

python
import triton.language.extra.cann.extension as extension

k_rows = tl.zeros((BLOCK_N, BLOCK_DMODEL), dtype=q.dtype)
for i in range(0, BLOCK_N):
    if start_n + i < split_kv_end:
        cache_row = extension.get_element(kv_locations, (i,))
        offsets = (
            cache_row * stride_k_token
            + kv_head * stride_k_head
            + tl.arange(0, BLOCK_DMODEL)
        )
        row = tl.load(K + offsets, mask=tl.arange(0, BLOCK_DMODEL) < key_dim)
        k_rows = extension.insert_slice(
            k_rows, row[None, :], (i, 0), (1, BLOCK_DMODEL), (1, 1)
        )
k = tl.trans(k_rows, (1, 0))
scores = tl.dot(q, k.to(q.dtype))

Ascend Guardrails

  • Prefer discrete outer rows with contiguous inner-vector loads; avoid forming a tile whose innermost access itself is indirect.
  • Use the extension namespace for get_element and insert_slice.
  • Mask the final KV tile and padded head dimensions independently.
  • Fit assembled K/KPE tiles and the attention accumulator in UB; reduce BLOCK_N or split feature dimensions when necessary.
  • Preserve float32 running maxima, sums, and accumulators in the online-softmax update.
  • Keep Q-head to KV-head grouping and split boundaries consistent with the original attention contract.

Verification

Compare with a PyTorch or trusted fused-attention reference across variable sequence lengths, KV splits, grouped-head ratios, optional positional feature dimensions, and partial BLOCK_N tails.

© Krusty84, Apache-2.0. Rendered from Markdown: HTML in the file is shown as text, images as links, and headings moved down two levels. Raw file

Files

Just SKILL.md in skills/optimize-triton-ascend-grouped-attention-access of Krusty84/triton-ascend-agent-dev-kit.

Open the folder on GitHubat commit 4ab5ee7

Compare with similar skills

Optimize Triton Ascend Grouped Attention Access next to the 5 skills that share the most tags, products or categories with it. Stars are the repository's; “used in” counts other GitHub owners with a copy.

Optimize Triton Ascend Grouped Attention Access compared with similar skills
SkillStarsUsed inTokensAuto-checkLicenceRepo updated
Optimize Triton Ascend Grouped Attention Access this skillKrusty84/triton-ascend-agent-dev-kit106—~658Automated safety check: PassApache-2.0
Agent BuildershareAI-lab/learn-claude-code78k4 repos~1.2kAutomated safety check: PassMIT
Add Uint Supportpytorch/pytorch104k2 repos~2.3kAutomated safety check: PassCustom licence
LLM Benchmarking with lm-evaluation-harnessOrchestra-Research/AI-Research-SKILLs13k8 repos~3kAutomated safety check: PassMIT
Segment Anything Model GuideOrchestra-Research/AI-Research-SKILLs13k8 repos~3.3kAutomated safety check: PassMIT
1passwordtrpc-group/trpc-agent-go1.9k14 repos~656Automated safety check: PassApache-2.0

Similar skills

  • Agent Builder

    shareAI-lab/learn-claude-code

    Design and build AI agents for any domain. An agent skill from shareAI-lab/learn-claude-code.

    78k GitHub starsUsed in 4 repos~1.2k tokens
    AI & LLM EngineeringAuto-check passed
  • Add Uint Support

    pytorch/pytorch

    Add unsigned integer (uint) type support to PyTorch operators by updating ATDISPATCH macros.

    104k GitHub starsUsed in 2 repos~2.3k tokens
    AI & LLM EngineeringAuto-check passed
  • LLM Benchmarking with lm-evaluation-harness

    Orchestra-Research/AI-Research-SKILLs

    Runs lm-evaluation-harness to benchmark language models on academic suites such as MMLU, GSM8K and HumanEval, compare models and track training checkpoints.

    13k GitHub starsUsed in 8 repos~3k tokens
    AI & LLM EngineeringAuto-check passed
  • Segment Anything Model Guide

    Orchestra-Research/AI-Research-SKILLs

    Guide to using Meta's Segment Anything Model for zero-shot image segmentation with point, box or mask prompts, or automatic mask generation.

    13k GitHub starsUsed in 8 repos~3.3k tokens
    AI & LLM EngineeringAuto-check passed
  • 1password

    trpc-group/trpc-agent-go

    Set up and use 1Password CLI (op). An agent skill from trpc-group/trpc-agent-go.

    1.9k GitHub starsUsed in 14 repos~656 tokens
    AI & LLM EngineeringAuto-check passed
  • Planning With Files

    jarrodwatts/claude-code-config

    Transforms workflow to use Manus-style persistent markdown files for planning, progress tracking, and knowledge storage.

    1.1k GitHub starsUsed in 5 repos~967 tokens
    AI & LLM EngineeringAuto-check passed

More from Krusty84/triton-ascend-agent-dev-kit

All 23 skills in this repo
  • Triton-Ascend Token Pool Assignment

    Krusty84/triton-ascend-agent-dev-kit

    Implements request-to-token-pool copies for Triton kernels on Ascend NPUs, replacing loop-carried vector offsets with fixed-offset block loops that compile reliably.

    106 GitHub stars~564 tokensUpdated 1 mo ago
    Auto-check passed
  • Triton-Ascend Batch Token Reorder

    Krusty84/triton-ascend-agent-dev-kit

    Writes a Triton kernel pattern for Ascend NPU that gathers several indexed token rows per program into an on-chip buffer and stores one contiguous output tile, for MoE-style token reordering.

    106 GitHub stars~597 tokensUpdated 1 mo ago
    Auto-check passed
  • Triton-Ascend Binned MoE Routing

    Krusty84/triton-ascend-agent-dev-kit

    Builds Triton-Ascend kernels that pack sorted token assignments into fixed expert-capacity tensors, restore top-k outputs and compute router-weight gradients on Ascend NPUs.

    106 GitHub stars~662 tokensUpdated 1 mo ago
    Auto-check passed
  • Triton-Ascend Fused Attention

    Krusty84/triton-ascend-agent-dev-kit

    Guides building a FlashAttention-v2-style fused forward attention kernel for Triton-Ascend on Ascend NPU, with online softmax, causal staging and float32 accumulation.

    106 GitHub stars~753 tokensUpdated 1 mo ago
    Auto-check passed
  • Triton-Ascend Fused Softmax

    Krusty84/triton-ascend-agent-dev-kit

    Builds a fused row-wise softmax kernel for Triton-Ascend that reads and writes each row once, handling padding, strides and masked loads on Ascend NPUs.

    106 GitHub stars~636 tokensUpdated 1 mo ago
    Auto-check passed
  • Build Triton Ascend Layer Norm

    Krusty84/triton-ascend-agent-dev-kit

    Build a fused forward LayerNorm kernel for Triton-Ascend with row-wise mean and variance reductions, float32 accumulation, affine weight and bias, and masked feature tiles.

    106 GitHub stars~659 tokensUpdated 1 mo ago
    Auto-check passed

Questions about Optimize Triton Ascend Grouped Attention Access

What does Optimize Triton Ascend Grouped Attention Access do?

Optimize irregular KV-cache access in Triton-Ascend grouped decode attention by vector-loading contiguous inner dimensions, assembling discrete outer rows with CANN slice extensions, and transposing…. Optimize Triton Ascend Grouped Attention Access is an agent skill from Krusty84/triton-ascend-agent-dev-kit.dot.

When should I use Optimize Triton Ascend Grouped Attention Access?

Optimize Triton Ascend Grouped Attention Access fits situations like: an agent ports grouped-query decode attention whose K; V cache addresses are indirect and naive low-axis-discrete loads degrade to scalar memory access.

How do I install Optimize Triton Ascend Grouped Attention Access in Claude Code?

Run `npx skills add Krusty84/triton-ascend-agent-dev-kit --skill optimize-triton-ascend-grouped-attention-access -a claude-code`. Or copy the skill folder (skills/optimize-triton-ascend-grouped-attention-access in Krusty84/triton-ascend-agent-dev-kit) into .claude/skills/optimize-triton-ascend-grouped-attention-access in your project. Claude Code loads it when a task matches its description.

How do I install Optimize Triton Ascend Grouped Attention Access in Codex?

Run `npx skills add Krusty84/triton-ascend-agent-dev-kit --skill optimize-triton-ascend-grouped-attention-access -a codex`. Or copy the skill folder (skills/optimize-triton-ascend-grouped-attention-access in Krusty84/triton-ascend-agent-dev-kit) into .agents/skills/optimize-triton-ascend-grouped-attention-access in your project. Codex loads it when a task matches its description.

Can I use Optimize Triton Ascend Grouped Attention Access in Cursor, Gemini CLI or GitHub Copilot?

Cursor, Gemini CLI, GitHub Copilot and OpenCode also load SKILL.md folders. With the skills CLI, run `npx skills add Krusty84/triton-ascend-agent-dev-kit --skill optimize-triton-ascend-grouped-attention-access -a cursor` (or -a gemini-cli, github-copilot or opencode for the others). To copy it by hand, put the folder in .cursor/skills/optimize-triton-ascend-grouped-attention-access, .gemini/skills/optimize-triton-ascend-grouped-attention-access, .github/skills/optimize-triton-ascend-grouped-attention-access and .opencode/skills/optimize-triton-ascend-grouped-attention-access in your project.

What does Optimize Triton Ascend Grouped Attention Access need to run?

SKILL.md names no scripts, command-line tools or credentials: Optimize Triton Ascend Grouped Attention Access is instructions for the agent only. Our summary lists: Python 3.

Does Optimize Triton Ascend Grouped Attention Access access the network?

SKILL.md contains no URLs. Any network use would come from the scripts or tools the agent runs. This is read from the text; nothing was executed.

Is Optimize Triton Ascend Grouped Attention Access safe to install?

Our automated static check of SKILL.md found no risky patterns, such as piping downloads into a shell, reading credential files or hidden Unicode. It is not a guarantee. Review the folder before installing.

What licence does Optimize Triton Ascend Grouped Attention Access use?

Optimize Triton Ascend Grouped Attention Access is published under the Apache-2.0 licence (the repository's licence). It allows redistribution, so the full SKILL.md is shown on this page.

How many tokens does Optimize Triton Ascend Grouped Attention Access use?

About 658 tokens (SKILL.md is roughly 2.6k characters). Agents keep only the skill's name and description in context until a task matches; then they load SKILL.md in full.

What are the alternatives to Optimize Triton Ascend Grouped Attention Access?

Skills that share tags, products or a category with Optimize Triton Ascend Grouped Attention Access: Agent Builder (shareAI-lab/learn-claude-code, 78k stars), Add Uint Support (pytorch/pytorch, 104k stars), LLM Benchmarking with lm-evaluation-harness (Orchestra-Research/AI-Research-SKILLs, 13k stars) and Segment Anything Model Guide (Orchestra-Research/AI-Research-SKILLs, 13k stars). The comparison table on this page puts their stars, adoption, token cost, safety result and licence side by side.

Who maintains Optimize Triton Ascend Grouped Attention Access?

Krusty84 (a GitHub user) maintains it in Krusty84/triton-ascend-agent-dev-kit, which has 106 GitHub stars. The repository holds 23 skills in this directory. The repository was last updated on August 15, 2026.

Source: Krusty84/triton-ascend-agent-dev-kit on GitHub. Facts on this page come from the repository at the commit we read; the author's words are quoted as theirs.